from tethys_datasets.engines import CkanDatasetEngine


ckan_engine = CkanDatasetEngine(endpoint="http://ciwweb.chpc.utah.edu/api/3/action",
                                apikey='003654e6-cd89-46a6-9035-28e4037b44d6')
# ckan_engine = CKANDatasetEngine(api_endpoint="http://tethys.byu.edu/api/3/action",
#                                 apikey='8cd924a8-48d3-452a-becf-d5ec19e9801b')


ckan_engine.list_datasets(limit=5, offset=1, console=True)
# ckan_engine.list_datasets(with_resources=True, limit=1, offset=1, console=True)
# ckan_engine.get_dataset('new_dataset', console=True)
# ckan_engine.get_resource('08929eb9-fc3b-4af3-baeb-417e1135fcdc', console=True)
# ckan_engine.search_resources(query={'model': 'GSSHA', 'name': 'Test_2'}, console=True)
# ckan_engine.search_datasets(console=True, query={'name': 'azure-blob-test'})

# ckan_engine.create_dataset(name='newest-dataset',
#                            notes='This is the newest dataset.',
#                            title='Newest Dataset',
#                            author='Me',
#                            console=True)

# ckan_engine.update_dataset(dataset_id='newest-dataset',
#                            notes='This is a test.',
#                            version='3.0',
#                            author='Someone Else',
#                            console=True)

# ckan_engine.update_resource(resource_id='d4537829-5024-402a-a591-9c077f9eb844',
#                             file='/Users/swainn/testing/timeseries_maps/ParkCityBasic/write/post_gis/ele_png.kmz',
#                             format='kmz',
#                             new_field='new_value',
#                             console=True)

# ckan_engine.delete_dataset(dataset_id='newest-dataset',
#                            console=True)

# ckan_engine.delete_resource(resource_id='0032994e-bc2c-47bc-8d32-17d536e69d33',
#                             console=True)

# ckan_engine.create_resource(dataset_id='new_dataset',
#                             file='/Users/swainn/testing/test_models/ParkCityBasic.zip',
#                             name='Park City Basic',
#                             model='GSSHA',
#                             format='zip',
#                             console=True)

# ckan_engine.create_resource(dataset_id='new_dataset',
#                             file='/Users/swainn/testing/test_models/ParkCityBasic/luse.idx',
#                             format='txt',
#                             mimetype='application/octet-stream',
#                             console=True)


# ckan_engine.create_resource(dataset_id='new_dataset',
#                             url='http://ciwweb.chpc.utah.edu/dataset/6c354b2c-2618-4529-8797-27235f1f5bc3/resource/08929eb9-fc3b-4af3-baeb-417e1135fcdc/download/parkcity.kml',
#                             format='kml',
#                             console=True)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tethys_datasets', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datasetservice',
            name='engine',
            field=models.CharField(default=b'tethys_dataset_services.engines.CkanDatasetEngine', max_length=200, choices=[(b'tethys_dataset_services.engines.CkanDatasetEngine', b'CKAN'), (b'tethys_dataset_services.engines.HydroShareDatasetEngine', b'HydroShare')]),
            preserve_default=True,
        ),
    ]



import os
import random
import string
import unittest

from ..engines import CkanDatasetEngine

try:
    from .test_config import TEST_CKAN_DATASET_SERVICE

except ImportError:
    print('ERROR: To perform tests, you must create a file in the "tests" package called "test_config.py". In this file'
          'provide a dictionary called "TEST_CKAN_DATASET_SERVICE" with keys "API_ENDPOINT" and "APIKEY".')
    exit(1)


def random_string_generator(size):
    chars = string.ascii_lowercase + string.digits
    return ''.join(random.choice(chars) for _ in range(size))


class TestCkanDatasetEngine(unittest.TestCase):

    def setUp(self):
        # Create Test Engine
        self.engine = CkanDatasetEngine(endpoint=TEST_CKAN_DATASET_SERVICE['ENDPOINT'],
                                        apikey=TEST_CKAN_DATASET_SERVICE['APIKEY'])

        # Create Test Dataset
        self.test_dataset_name = random_string_generator(10)
        dataset_result = self.engine.create_dataset(name=self.test_dataset_name, version='1.0')
        self.test_dataset = dataset_result['result']

        # Create Test Resource
        self.test_resource_name = random_string_generator(10)
        self.test_resource_url = 'http://home.byu.edu'
        resource_result = self.engine.create_resource(self.test_dataset_name, url=self.test_resource_url, format='zip')
        self.test_resource = resource_result['result']

    def tearDown(self):
        # Delete test resource and dataset
        self.engine.delete_resource(resource_id=self.test_resource['id'])
        self.engine.delete_dataset(dataset_id=self.test_dataset_name)

    def test_list_datasets_defaults(self):
        # Execute
        result = self.engine.list_datasets()

        # Verify Success
        self.assertTrue(result['success'])

    def test_list_datasets_with_resources(self):
        # Execute
        result = self.engine.list_datasets(with_resources=True)

        # Verify Success
        self.assertTrue(result['success'])

    def test_list_datasets_with_params(self):
        # Setup
        limit = 5
        number_all = len(self.engine.list_datasets()['result'])

        # Execute twice with offsets different
        result_page_1 = self.engine.list_datasets(limit=limit, offset=1)
        result_page_2 = self.engine.list_datasets(limit=limit, offset=2)

        # Verify success
        self.assertTrue(result_page_1['success'])
        self.assertTrue(result_page_2['success'])

        # Count the results
        page_1_count = len(result_page_1['result'])
        page_2_count = len(result_page_2['result'])

        # Verify count (should be less than or equal to limit)
        self.assertLessEqual(page_1_count, limit)
        self.assertLessEqual(page_2_count, limit)

        # If there are more than 5 datasets, the results should be different
        if number_all > 5:
            self.assertNotEqual(result_page_1, result_page_2)

    def test_search_resources(self):
        # Execute
        result = self.engine.search_resources(query={'format': 'zip'})

        # Verify Success
        self.assertTrue(result['success'])

        # Check search results if they exist
        search_results = result['result']['results']

        if len(search_results) > 1:
            for result in search_results:
                self.assertIn('zip', result['format'].lower())

    def test_search_datasets(self):
        # Execute
        result = self.engine.search_datasets(query={'version': '1.0'})

        # Verify Success
        self.assertTrue(result['success'])

        # Check search results if they exist
        search_results = result['result']['results']

        if len(search_results) > 1:
            for result in search_results:
                self.assertIn('version', result['1.0'].lower())

    def test_create_dataset(self):
        # Setup
        new_dataset_name = random_string_generator(10)

        # Execute
        result = self.engine.create_dataset(name=new_dataset_name)

        # Verify Success
        self.assertTrue(result['success'])

        # Should return the new one
        self.assertEqual(new_dataset_name, result['result']['name'])

        # Delete
        self.engine.delete_dataset(dataset_id=new_dataset_name)

    def test_create_resource_url(self):
        # Setup
        new_resource_name = random_string_generator(5)
        new_resource_url = 'http://home.byu.edu'

        # Execute
        result = self.engine.create_resource(dataset_id=self.test_dataset_name,
                                             url=new_resource_url,
                                             name=new_resource_name)

        # Verify Success
        self.assertTrue(result['success'])

        # Verify name and url
        self.assertEqual(new_resource_name, result['result']['name'])
        self.assertEqual(new_resource_url, result['result']['url'])

        # Delete
        self.engine.delete_resource(resource_id=result['result']['id'])

    def test_create_resource_file_upload(self):
        # Prepare
        file_to_upload = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'support', 'upload_test.txt')

        # Execute
        result = self.engine.create_resource(dataset_id=self.test_dataset_name, file=file_to_upload)

        # Verify Success
        self.assertTrue(result['success'], result)

        # Verify name and url_type (which should be upload if file upload)
        self.assertEqual(result['result']['name'], 'upload_test.txt')
        self.assertEqual(result['result']['url_type'], 'upload')

        # Delete
        self.engine.delete_resource(resource_id=result['result']['id'])

    def test_get_dataset(self):
        # Execute
        result = self.engine.get_dataset(dataset_id=self.test_dataset_name)

        # Verify Success
        self.assertTrue(result['success'])

        # Verify Name
        self.assertEqual(result['result']['name'], self.test_dataset_name)

    def test_get_resource(self):
        # Execute
        result = self.engine.get_resource(resource_id=self.test_resource['id'])

        # Verify Success
        self.assertTrue(result['success'])

        # Verify Properties
        self.assertEqual(result['result']['url'], self.test_resource_url)

    def test_update_dataset(self):
        # Setup
        test_version = '2.0'

        # Execute
        result = self.engine.update_dataset(dataset_id=self.test_dataset_name, version=test_version)

        # Verify Success
        self.assertTrue(result['success'])

        # Verify new version property
        self.assertEqual(result['result']['version'], test_version)

    def test_update_resource_property_change(self):
        # Setup
        new_format = 'web'

        # Execute
        result = self.engine.update_resource(resource_id=self.test_resource['id'], format=new_format)

        # Verify Success
        self.assertTrue(result['success'])

        # Verify new format
        self.assertEqual(result['result']['format'], new_format)

    def test_update_resource_url_change(self):
        # Setup
        new_url = 'http://www.utah.edu'

        # Execute
        result = self.engine.update_resource(resource_id=self.test_resource['id'], url=new_url)

        # Verify Success
        self.assertTrue(result['success'])

        # Verify New URL Property
        self.assertEqual(result['result']['url'], new_url)
        self.assertNotEqual(result['result']['url'], self.test_resource['url'])

    def test_update_resource_file_upload(self):
        # Setup
        file_to_upload = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'support', 'upload_test.txt')

        # Execute
        result = self.engine.update_resource(resource_id=self.test_resource['id'], file=file_to_upload)

        # Verify Success
        self.assertTrue(result['success'])

        # Verify Name (should be the same as the file uploaded by default)
        self.assertEqual(result['result']['name'], 'upload_test.txt')

        # URL should be different than original when file upload executes
        self.assertNotEqual(result['result']['url'], self.test_resource['url'])

    def test_delete_resource(self):
        # Execute
        result = self.engine.delete_resource(resource_id=self.test_resource['id'])

        # Verify Success
        self.assertTrue(result['success'])

        # Delete requests should return nothing
        self.assertEqual(result['result'], None)

    def test_delete_dataset(self):
        # Execute
        result = self.engine.delete_dataset(dataset_id=self.test_dataset_name)

        # Confirm Success
        self.assertTrue(result['success'])

        # Delete requests should return nothing
        self.assertEqual(result['result'], None)


from tethys_apps.base.app_base import TethysAppBase
from tethys_dataset_services.valid_engines import VALID_ENGINES
from .models import DatasetService as DsModel


class DatasetService:
    """
    Used to define dataset services for apps.
    """

    def __init__(self, name, type, endpoint, apikey=None, username=None, password=None):
        """
        Constructor
        """
        self.name = name

        # Validate the types
        if type in VALID_ENGINES:
            self.type = type
            self.engine = VALID_ENGINES[type]
        else:
            if len(VALID_ENGINES) > 2:
                comma_separated_types = ', '.join('"{0}"'.format(t) for t in VALID_ENGINES.keys()[:-1])
                last_type = '"{0}"'.format(VALID_ENGINES.keys()[-1])
                valid_types_string = '{0}, and {1}'.format(comma_separated_types, last_type)
            elif len(VALID_ENGINES) == 2:
                valid_types_string = '"{0}" and "{1}"'.format(VALID_ENGINES.keys()[0], VALID_ENGINES.keys()[1])
            else:
                valid_types_string = '"{0}"'.format(VALID_ENGINES.keys()[0])

            raise ValueError('The value "{0}" is not a valid for argument "type" of DatasetService. Valid values for '
                             '"type" argument include {1}.'.format(type, valid_types_string))

        self.endpoint = endpoint
        self.apikey = apikey
        self.username = username
        self.password = password

    def __repr__(self):
        """
        String representation
        """
        return '<DatasetService: type={0}, api_endpoint={1}>'.format(self.type, self.endpoint)


def initialize_engine_object(engine, endpoint, apikey=None, username=None, password=None):
    """
    Initialize a DatasetEngine object from a string that points at the engine class.
    """
    # Derive import parts from engine string
    engine_split = engine.split('.')
    module_string = '.'.join(engine_split[:-1])
    engine_class_string = engine_split[-1]

    # Import
    module = __import__(module_string, fromlist=[engine_class_string])
    EngineClass = getattr(module, engine_class_string)

    # Create Engine Object
    engine_instance = EngineClass(endpoint=endpoint,
                                  apikey=apikey,
                                  username=username,
                                  password=password)
    return engine_instance


def get_dataset_engine(name, app_class=None):
    """
    Get a dataset engine with the given name.

    Args:
      name (string): Name of the dataset engine to retrieve.
      app_class (class): The app class to include in the search for dataset engines.

    Returns:
      (DatasetEngine): A dataset engine object.
    """
    # If the app_class is given, check it first for a dataset engine
    app_dataset_services = None

    if app_class and issubclass(app_class, TethysAppBase):
        # Instantiate app class and retrieve dataset services list
        app = app_class()
        app_dataset_services = app.dataset_services()

    if app_dataset_services:
        # Search for match
        for app_dataset_service in app_dataset_services:

            # If match is found, initiate engine object
            if app_dataset_service.name == name:
                return initialize_engine_object(engine=app_dataset_service.engine,
                                                endpoint=app_dataset_service.endpoint,
                                                apikey=app_dataset_service.apikey,
                                                username=app_dataset_service.username,
                                                password=app_dataset_service.password)

    # If the dataset engine cannot be found in the app_class, check database for site-wide dataset engines
    site_dataset_services = DsModel.objects.all()

    if site_dataset_services:
        # Search for match
        for site_dataset_service in site_dataset_services:

            # If match is found initiate engine object
            if site_dataset_service.name == name:
                dataset_service_object = initialize_engine_object(engine=site_dataset_service.engine.encode('utf-8'),
                                                                  endpoint=site_dataset_service.endpoint,
                                                                  apikey=site_dataset_service.apikey,
                                                                  username=site_dataset_service.username,
                                                                  password=site_dataset_service.password)

                return dataset_service_object

    raise NameError('Could not find dataset service with name "{0}". Please check that dataset service with that name '
                    'exists in settings.py or in your app.py.'.format(name))










from django.contrib import admin
from .models import DatasetService


class DatasetServiceAdmin(admin.ModelAdmin):
    """
    Admin model for Web Processing Service Model
    """
    fields = ('name', 'engine', 'apikey', 'endpoint', 'username', 'password')


admin.site.register(DatasetService, DatasetServiceAdmin)



from django.db import models
from tethys_dataset_services.valid_engines import VALID_ENGINES


# Create your models here.
class DatasetService(models.Model):
    """
    ORM for Dataset Service settings.
    """
    # Define default values for engine choices
    CKAN = VALID_ENGINES['ckan']
    HYDROSHARE = VALID_ENGINES['hydroshare']

    # Define default choices for engine selection
    ENGINE_CHOICES = (
        (CKAN, 'CKAN'),
        (HYDROSHARE, 'HydroShare')
    )

    name = models.CharField(max_length=30, unique=True)
    engine = models.CharField(max_length=200, choices=ENGINE_CHOICES, default=CKAN)
    endpoint = models.CharField(max_length=1024)
    apikey = models.CharField(max_length=100, blank=True)
    username = models.CharField(max_length=100, blank=True)
    password = models.CharField(max_length=100, blank=True)

    class Meta:
        verbose_name = 'Dataset Service'
        verbose_name_plural = 'Dataset Services'

    def __unicode__(self):
        return self.name
